#include <vcl.h>
#pragma hdrstop

#include "PreviewForm.h"

#pragma package(smart_init)
#pragma resource "*.dfm"

TPreviewForm *PreviewForm;

__fastcall TPreviewForm::TPreviewForm(TComponent* Owner)
  : TForm(Owner)
{
  PreviewControl->DeviceInfo->ReferenceHandle = Printer()->Handle;

  int line = 1;
  for (int i = 0; i < 6; i++) {
    TMetafile *meta = PreviewControl->Add();
    TMetafileCanvas *canvas = new TMetafileCanvas(meta,
        PreviewControl->DeviceInfo->ReferenceHandle);
    canvas->Font->PixelsPerInch = PreviewControl->DeviceInfo->LogPixelsY;
    canvas->Font->Size = 12;

    // Assume 8.5x11" paper with 1" margins
    const int margin = 1;
    for (int y = PreviewControl->DeviceInfo->LogPixelsY * margin;
         y < (11 - margin) * PreviewControl->DeviceInfo->LogPixelsY;
         y += canvas->TextHeight("Line")) {
      canvas->TextOut(margin * PreviewControl->DeviceInfo->LogPixelsX, y,
          String().sprintf(L"Line %i", line++));
    }

    delete canvas;
  }

  ActiveControl = PrintButton;
}

__fastcall TPreviewForm::~TPreviewForm()
{
}

void __fastcall TPreviewForm::FormShow(TObject * /*Sender*/)
{
  Caption = "Sample Report";

  ShowInitialPage();
}

/**Finishes setting up the preview control and page navigation and shows the
 * first page.
 */
void TPreviewForm::ShowInitialPage()
{
  PreviewControl->First();
  ShowPageNumber(0);
  PageCountLabel->Caption = String().sprintf(L"of %d", PreviewControl->PageCount);
}

void __fastcall TPreviewForm::FormResize(TObject * /*Sender*/)
{
  OnResize = NULL;  // Execute this code on first resize only.

  // Use 100% zoom if it will fit.  For smaller displays, zoom out so the page
  // width will fit.
  //
  // This relies on PreviewControl->Options->ScaleMode = smPageWidth in the
  // designer.
	if (PreviewControl->Options->Scale < 100) {
	  PageWidthButtonClick(this);
	} else {
	  PreviewControl->Options->ScaleMode = smScale;
	  PreviewControl->Options->Scale = 100;
	}
}

void __fastcall TPreviewForm::PrintButtonClick(TObject * /*Sender*/)
{
  Application->MessageBox(L"Print!", L"");
}

void __fastcall TPreviewForm::PdfButtonClick(TObject * /*Sender*/)
{
  Application->MessageBox(L"PDF!", L"");
}

void __fastcall TPreviewForm::FirstButtonClick(TObject * /*Sender*/)
{
	PreviewControl->First();
	ShowPageNumber(0);
}

void __fastcall TPreviewForm::PrevButtonClick(TObject * /*Sender*/)
{
	PreviewControl->Prior();
	ShowPageNumber(mIndex - 1);
}

void __fastcall TPreviewForm::NextButtonClick(TObject * /*Sender*/)
{
	PreviewControl->Next();
	ShowPageNumber(mIndex + 1);
}

void __fastcall TPreviewForm::LastButtonClick(TObject * /*Sender*/)
{
	PreviewControl->Last();
	ShowPageNumber(PreviewControl->PageCount - 1);
}

void __fastcall TPreviewForm::CloseButtonClick(TObject * /*Sender*/)
{
	this->Close();
}

void __fastcall TPreviewForm::FullPageButtonClick(TObject * /*Sender*/)
{
	FullPageButton->Down = true;
	PageWidthButton->Down = false;
	PreviewControl->Options->ScaleMode = smFullPage;
	UpdateScaleDisplay(PreviewControl->Options->Scale);
	PreviewControl->TopRow = mIndex;
}

void __fastcall TPreviewForm::PageWidthButtonClick(TObject * /*Sender*/)
{
	FullPageButton->Down = false;
	PageWidthButton->Down = true;
	PreviewControl->Options->ScaleMode = smPageWidth;
	UpdateScaleDisplay(PreviewControl->Options->Scale);
	PreviewControl->TopRow = mIndex;
}

void __fastcall TPreviewForm::ScaleComboBoxChange(TObject * /*Sender*/)
{
	FullPageButton->Down = false;
	PageWidthButton->Down = false;
	PreviewControl->Options->ScaleMode = smScale;
	PreviewControl->Options->Scale = atoi(AnsiString(ScaleComboBox->Text).c_str());
	PreviewControl->TopRow = mIndex;
}

void __fastcall TPreviewForm::ScaleComboBoxExit(TObject * /*Sender*/)
{
  UpdateScaleDisplay(atoi(AnsiString(ScaleComboBox->Text).c_str()));
}

void TPreviewForm::UpdateScaleDisplay(int new_scale)
{
  ScaleComboBox->Text = IntToStr(new_scale) + "%";
}

void __fastcall TPreviewForm::PreviewControlAfterScroll(TObject * /*Sender*/)
{
	mIndex = PreviewControl->TopRow;
	ShowPageNumber(mIndex);
}

void __fastcall TPreviewForm::PageEditKeyPress(TObject * /*Sender*/, char &Key)
{
	if (Key == '\r') {
		int new_index = mIndex;
		try {
			new_index = StrToInt(PageEdit->Text);
		}	catch (Exception& e) {
			Beep();
		}
		if (new_index < 1) {
			mIndex = 0;
		} else if (new_index >= PreviewControl->PageCount) {
			mIndex = PreviewControl->PageCount - 1;
		} else {
			mIndex = new_index - 1;
		}
		PreviewControl->TopRow = mIndex;
		ShowPageNumber(mIndex);
		Key = '\0';
	}
}

void TPreviewForm::ShowPageNumber(int index)
{
	PageEdit->Text = IntToStr(index + 1);
	mIndex = index;
	FirstButton->Enabled = false;
	PrevButton->Enabled = false;
	NextButton->Enabled = false;
	LastButton->Enabled = false;
	if (index > 0) {
		FirstButton->Enabled = true;
		PrevButton->Enabled = true;
	}
	if (index < PreviewControl->PageCount - 1) {
		NextButton->Enabled = true;
		LastButton->Enabled = true;
	}
}

void __fastcall TPreviewForm::SaveCsvMenuItemClick(TObject * /*Sender*/)
{
  Application->MessageBox(L"CSV!", L"");
}

void __fastcall TPreviewForm::CopyMenuItemClick(TObject * /*Sender*/)
{
  Application->MessageBox(L"Clipboard!", L"");
}

void __fastcall TPreviewForm::CopyImageMenuItemClick(TObject * /*Sender*/)
{
  Application->MessageBox(L"Copy image!", L"");
}

void __fastcall TPreviewForm::NotesButtonClick(TObject * /*Sender*/)
{
  Application->MessageBox(L"Notes!", L"");
}

void __fastcall TPreviewForm::FormKeyDown(TObject * /*Sender*/, WORD &Key,
    TShiftState /*Shift*/)
{
  switch (Key) {
    case VK_ESCAPE:
      CloseButton->Click();
      Key = 0;
      break;
    case VK_NEXT:
      SendMessage(PreviewControl->Handle, WM_VSCROLL, SB_PAGEDOWN, 0);
      Key = 0;
      break;
    case VK_PRIOR:
      SendMessage(PreviewControl->Handle, WM_VSCROLL, SB_PAGEUP, 0);
      Key = 0;
      break;
    case VK_UP:
      // This also relies on our CMDialogKey below.
      SendMessage(PreviewControl->Handle, WM_VSCROLL, SB_LINEUP, 0);
      Key = 0;
      break;
    case VK_DOWN:
      // This also relies on our CMDialogKey below.
      SendMessage(PreviewControl->Handle, WM_VSCROLL, SB_LINEDOWN, 0);
      Key = 0;
      break;
  }
}

void __fastcall TPreviewForm::FormKeyPress(TObject * /*Sender*/,
    System::WideChar &Key)
{
  if (Key == ('W' & 037)) {
    CloseButton->Click();
    Key = 0;
  }
}

/**This is used to ensure that our OnKeyDown event gets dialog navigation
 * keys.  See http://stackoverflow.com/a/15189692/25507
 */
void __fastcall TPreviewForm::CMDialogKey(Winapi::Messages::TWMKey &Message)
{
  if (Message.CharCode == VK_UP || Message.CharCode == VK_DOWN) {
    Message.Result = 0;
  } else {
    TForm::Dispatch(&Message);
  }
}
